// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#ifdef __USE_OSX_IMPLEMENTATION__
#include <strings.h>
#include "missing.hpp"
#include <Carbon/Carbon.h>
#undef check
#endif

#include <boost/python/class.hpp>
#include <boost/python/return_value_policy.hpp>

#include <Producer/Referenced>
#include <Producer/CameraConfig>

#include "producer_ptr.hpp"

using namespace boost::python;

namespace PyProd {
void init_CameraConfig()
{
    class_<Producer::CameraConfig, Producer::ref_ptr<Producer::CameraConfig>, bases<Producer::Referenced>, boost::noncopyable> camera_config("CameraConfig", no_init);

    camera_config
        .def(init<>())
        .def("beginVisual", 
            (void (Producer::CameraConfig::*)(void)) &Producer::CameraConfig::beginVisual)
        .def("beginVisual", 
            (void (Producer::CameraConfig::*)(const char *)) &Producer::CameraConfig::beginVisual)
        .def("setVisualSimpleConfiguration", 
            &Producer::CameraConfig::setVisualSimpleConfiguration)
        .def("setVisualByID", 
            &Producer::CameraConfig::setVisualByID)
        .def("endVisual", &Producer::CameraConfig::endVisual)

        .def("findVisual", &Producer::CameraConfig::findVisual, return_internal_reference<>())

        .def("parseFile", &Producer::CameraConfig::parseFile)

        .def("beginRenderSurface", &Producer::CameraConfig::beginRenderSurface)

        .def("setRenderSurfaceVisualChooser", (void (Producer::CameraConfig::*)(const char *))
                &Producer::CameraConfig::setRenderSurfaceVisualChooser)
        .def("setRenderSurfaceVisualChooser", (void (Producer::CameraConfig::*)(void))
                &Producer::CameraConfig::setRenderSurfaceVisualChooser)
        .def("setRenderSurfaceWindowRectangle", &Producer::CameraConfig::setRenderSurfaceWindowRectangle)

        .def("setRenderSurfaceHostName", &Producer::CameraConfig::setRenderSurfaceHostName)

        .def("setRenderSurfaceDisplayNum", &Producer::CameraConfig::setRenderSurfaceDisplayNum)

        .def("setRenderSurfaceScreen", &Producer::CameraConfig::setRenderSurfaceScreen)

        .def("setRenderSurfaceBorder", &Producer::CameraConfig::setRenderSurfaceBorder)

        .def("setRenderSurfaceInputRectangle", &Producer::CameraConfig::setRenderSurfaceInputRectangle)

        .def("endRenderSurface", &Producer::CameraConfig::endRenderSurface)

        .def("findRenderSurface", &Producer::CameraConfig::findRenderSurface, return_internal_reference<>())

        .def("getNumberOfRenderSurfaces", &Producer::CameraConfig::getNumberOfRenderSurfaces)

        .def("getRenderSurface", &Producer::CameraConfig::getRenderSurface, return_internal_reference<>())
        
        // addVisualAttribute TODO
        // addVisualExtendedAttribute TODO

        .def("addCamera", 
            &Producer::CameraConfig::addCamera)
        .def("beginCamera", &Producer::CameraConfig::beginCamera)
        
        .def("setCameraRenderSurface", (void (Producer::CameraConfig::*)(const char *))
                &Producer::CameraConfig::setCameraRenderSurface)
        .def("setCameraRenderSurface", (void (Producer::CameraConfig::*)(void))
                &Producer::CameraConfig::setCameraRenderSurface)

        .def("setCameraProjectionRectangle", (void (Producer::CameraConfig::*)(float, float, float, float))
                &Producer::CameraConfig::setCameraProjectionRectangle)

        .def("setCameraProjectionRectangle", (void (Producer::CameraConfig::*)(int, int, int, int))
                &Producer::CameraConfig::setCameraProjectionRectangle)

        .def("setCameraOrtho", &Producer::CameraConfig::setCameraOrtho)

        .def("setCameraPerspective", &Producer::CameraConfig::setCameraPerspective)

        .def("setCameraFrustum", &Producer::CameraConfig::setCameraFrustum)

        .def("setCameraLensShear", &Producer::CameraConfig::setCameraLensShear)

        .def("beginCameraOffset", &Producer::CameraConfig::beginCameraOffset)

        .def("rotateCameraOffset", &Producer::CameraConfig::rotateCameraOffset)

        .def("translateCameraOffset", &Producer::CameraConfig::translateCameraOffset)

        .def("scaleCameraOffset", &Producer::CameraConfig::scaleCameraOffset)

        .def("shearCameraOffset", &Producer::CameraConfig::shearCameraOffset)

        .def("endCameraOffset", &Producer::CameraConfig::endCameraOffset)

        .def("endCamera", &Producer::CameraConfig::endCamera)

        .def("findCamera", &Producer::CameraConfig::findCamera, return_internal_reference<>())

        .def("getNumberOfCameras", &Producer::CameraConfig::getNumberOfCameras)

        .def("getCamera", 
            (Producer::Camera* (Producer::CameraConfig::*)(unsigned int))
            &Producer::CameraConfig::getCamera,
            return_value_policy<manage_producer_object>())

        .def("beginInputArea", &Producer::CameraConfig::beginInputArea)
        .def("addInputAreaEntry", &Producer::CameraConfig::addInputAreaEntry)
        .def("endInputArea", &Producer::CameraConfig::endInputArea)
        
        .def("setInputArea", &Producer::CameraConfig::setInputArea)
        .def("getInputArea", (Producer::InputArea * (Producer::CameraConfig::*)())
                &Producer::CameraConfig::getInputArea, return_internal_reference<>())

        .def("setInputArea", 
            &Producer::CameraConfig::setInputArea)

        .def("setInputArea", 
            &Producer::CameraConfig::setInputArea)

        .def("getInputArea", 
            (Producer::InputArea* (Producer::CameraConfig::*)()) &Producer::CameraConfig::getInputArea,
            return_value_policy<manage_producer_object>())

        .def("realize", 
            &Producer::CameraConfig::realize)

        .def("defaultConfig", 
            &Producer::CameraConfig::defaultConfig)

#if 0
        struct StereoSystemCommand
        {
            int _screen;
            std::string _setStereoCommand;
            std::string _restoreMonoCommand;

            StereoSystemCommand(int screen, std::string setStereoCommand, std::string restoreMonoCommand ):
                _screen(screen),
                _setStereoCommand(setStereoCommand),
                _restoreMonoCommand(restoreMonoCommand) {}
        };

        static std::string findFile( std::string );

        void addStereoSystemCommand( int screen, std::string stereoCmd, std::string monoCmd )
        {
            _stereoSystemCommands.push_back(StereoSystemCommand( screen, stereoCmd, monoCmd ));
        }

        const std::vector<StereoSystemCommand> &getStereoSystemCommands() { return _stereoSystemCommands; }
#endif
        ;

}
}
