import sys
from PyOSG import osg
from PyOSG import osgDB
from PyOSG import osgGA
import OpenGL   # for GL_LIGHTING

#
# A simple demo demonstrating different texturing modes, 
# including using of texture extensions.
#

# create quad at specified position.
def  createSquare(corner,width,height, image=None):
    # set up the Geometry.
    geom = osg.Geometry()

    coords = osg.Vec3Array(4)
    coords[0] = corner
    coords[1] = corner+width
    coords[2] = corner+width+height
    coords[3] = corner+height


    geom.setVertexArray(coords)

    norms = osg.Vec3Array(1)
    norms[0] = width^height
    norms[0].normalize()
    
    geom.setNormalArray(norms)
    geom.setNormalBinding(osg.Geometry.BIND_OVERALL)

    tcoords = osg.Vec2Array(4)
    tcoords[0].set(0.0,0.0);
    tcoords[1].set(1.0,0.0);
    tcoords[2].set(1.0,1.0);
    tcoords[3].set(0.0,1.0);
    geom.setTexCoordArray(0,tcoords)
    
    geom.addPrimitiveSet(osg.DrawArrays(osg.PrimitiveSet.QUADS,0,4))
    
    if (image):
        stateset = osg.StateSet()
        texture = osg.Texture2D()
        texture.setImage(image)
        stateset.setTextureAttributeAndModes(0,texture,osg.StateAttribute.ON)
        geom.setStateSet(stateset)

    return geom

def createAxis(corner, xdir, ydir, zdir):
    # set up the Geometry.
    geom = osg.Geometry();

    coords = osg.Vec3Array(6);
    coords[0] = corner;
    coords[1] = corner+xdir;
    coords[2] = corner;
    coords[3] = corner+ydir;
    coords[4] = corner;
    coords[5] = corner+zdir;

    geom.setVertexArray(coords);

    x_color = osg.Vec4(0.0,1.0,1.0,1.0)
    y_color = osg.Vec4(0.0,1.0,1.0,1.0)
    z_color = osg.Vec4(1.0,0.0,0.0,1.0)

    color = osg.Vec4Array(6);
    color[0] = x_color;
    color[1] = x_color;
    color[2] = y_color;
    color[3] = y_color;
    color[4] = z_color;
    color[5] = z_color;
    
    geom.setColorArray(color);
    geom.setColorBinding(osg.Geometry.BIND_PER_VERTEX);
    
    geom.addPrimitiveSet(osg.DrawArrays(osg.PrimitiveSet.LINES,0,6));
    
    stateset = osg.StateSet();
    linewidth = osg.LineWidth();
    linewidth.setWidth(4.0);
    stateset.setAttributeAndModes(linewidth,osg.StateAttribute.ON);
    stateset.setMode(OpenGL.GL.GL_LIGHTING,osg.StateAttribute.OFF);
    geom.setStateSet(stateset);
    
    return geom

def createModel():
    # create the root node which will hold the model.
    root = osg.Group();

    # add the drawable into a single goede to be shared...
    center = osg.Billboard();
    center.setMode(osg.Billboard.POINT_ROT_EYE);
    center.addDrawable(
        createSquare(osg.Vec3(-0.5,0.0,-0.5),osg.Vec3(1.0,0.0,0.0),osg.Vec3(0.0,0.0,1.0),osgDB.readImageFile("reflect.rgb")),
        osg.Vec3(0.0,0.0,0.0));
        
    x_arrow = osg.Billboard();
    x_arrow.setMode(osg.Billboard.AXIAL_ROT);
    x_arrow.setAxis(osg.Vec3(1.0,0.0,0.0));
    x_arrow.setNormal(osg.Vec3(0.0,-1.0,0.0));
    x_arrow.addDrawable(
        createSquare(osg.Vec3(-0.5,0.0,-0.5),osg.Vec3(1.0,0.0,0.0),osg.Vec3(0.0,0.0,1.0),osgDB.readImageFile("osg_posx.png")),
        osg.Vec3(5.0,0.0,0.0));

    y_arrow = osg.Billboard();
    y_arrow.setMode(osg.Billboard.AXIAL_ROT);
    y_arrow.setAxis(osg.Vec3(0.0,1.0,0.0));
    y_arrow.setNormal(osg.Vec3(1.0,0.0,0.0));
    y_arrow.addDrawable(
        createSquare(osg.Vec3(0.0,-0.5,-0.5),osg.Vec3(0.0,1.0,0.0),osg.Vec3(0.0,0.0,1.0),osgDB.readImageFile("osg_posy.png")),
        osg.Vec3(0.0,5.0,0.0));

    z_arrow = osg.Billboard();
    z_arrow.setMode(osg.Billboard.AXIAL_ROT);
    z_arrow.setAxis(osg.Vec3(0.0,0.0,1.0));
    z_arrow.setNormal(osg.Vec3(0.0,-1.0,0.0));
    z_arrow.addDrawable(
        createSquare(osg.Vec3(-0.5,0.0,-0.5),osg.Vec3(1.0,0.0,0.0),osg.Vec3(0.0,0.0,1.0),osgDB.readImageFile("osg_posz.png")),
        osg.Vec3(0.0,0.0,5.0))



    axis = osg.Geode()
    axis.addDrawable(createAxis(osg.Vec3(0.0,0.0,0.0),osg.Vec3(5.0,0.0,0.0),osg.Vec3(0.0,5.0,0.0),osg.Vec3(0.0,0.0,5.0)));


    root.addChild(center)
    root.addChild(x_arrow)
    root.addChild(y_arrow)
    root.addChild(z_arrow)
    root.addChild(axis)

    return root

def init(group):
    print dir(sys)
    rootNode = createModel()
    group.addChild(rootNode)

