#!/usr/bin/env python

from random import Random
import math
import sys
from PyOSG import Producer
from PyOSG import osg
from PyOSG import osgDB
from PyOSG import osgGA
from PyOSG import osgUtil
from PyOSG import osgProducer
from PyOSG import osgText
from PyOSG import osgSim
from OpenGL.GL import *
from TestManipulator import TestManipulator

# container storing all house nodes
nodes = []

Root = None

HOUSES_SIZE = 25000      # total number of houses
XDim = 5000.0            # area dimension +/- XDim
ZDim = 5000.0            # area dimension +/- YDim

GridX = 20               # number of grids in x direction
GridY = 20               # number of grids in y direction

UseImpostor = True      # use impostor (or do not use)

Threshold = 3000.0       # distance where impostor are shown

# create houses and store nodes in container
def CreateHouses():
    print "CreateHouses"
    global nodes
    indices = (
        0, 2, 1,
        3, 2, 0,
        0, 4, 7,
        7, 3, 0,
        0, 1, 5,
        5, 4, 0,
        1, 6, 5,
        2, 6, 1,
        2, 3, 7,
        2, 7, 6,
        4, 8, 7,
        5, 6, 9,
        4, 5, 8,
        8, 5, 9,
        6, 7, 8,
        8, 9, 6
    )

    # use the same color, normal and indices for all houses.
    colors = osg.Vec4Array(1)
    colors[0] = osg.Vec4(1.0, 1.0, 1.0, 1.0)

    # normals
    normals = osg.Vec3Array(16)
    normals[0] = osg.Vec3( 0.0,  -0.0, -1.0)
    normals[1] = osg.Vec3( 0.0,  -0.0, -1.0)
    normals[2] = osg.Vec3( 0.0,  -1.0,  0.0)
    normals[3] = osg.Vec3( 0.0,  -1.0,  0.0)
    normals[4] = osg.Vec3( 1.0,  -0.0,  0.0)
    normals[5] = osg.Vec3( 1.0,  -0.0,  0.0)
    normals[6] = osg.Vec3( 0.0, 1.0,  0.0)
    normals[7] = osg.Vec3( 0.0, 1.0,  0.0)
    normals[8] = osg.Vec3(-1.0,  -0.0,  0.0)
    normals[9] = osg.Vec3(-1.0,  -0.0,  0.0)
    normals[10] = osg.Vec3( 0.0,  -0.928477, 0.371391)
    normals[11] = osg.Vec3( 0.0, 0.928477, 0.371391)
    normals[12] = osg.Vec3( 0.707107,  0.0, 0.707107)
    normals[13] = osg.Vec3( 0.707107,  0.0, 0.707107)
    normals[14] = osg.Vec3(-0.707107,  0.0, 0.707107)
    normals[15] = osg.Vec3(-0.707107,  0.0, 0.707107)

    # coordIndices
    coordIndices = osg.UByteArray(indices)

    # share the primtive set.
    primitives = osg.DrawArrays(osg.PrimitiveSet.TRIANGLES,0,48)

    rand = Random().random
    for q in range(HOUSES_SIZE):
        if q%1000 == 0: print q,
        if q%100 == 0:  print ".",
        xPos = rand() * 2.0 * XDim - XDim

        yPos = rand() * 2.0 * ZDim - ZDim

        scale = 10.0

        offset = osg.Vec3(xPos,yPos,0.0)

        # coords
        coords = osg.Vec3Array(10)
        coords[0] = osg.Vec3( 0.5, -0.7, 0.0)
        coords[1] = osg.Vec3( 0.5,  0.7, 0.0)
        coords[2] = osg.Vec3(-0.5, 0.7, 0.0)
        coords[3] = osg.Vec3(-0.5, -0.7, 0.0)
        coords[4] = osg.Vec3( 0.5, -0.7, 1.0)
        coords[5] = osg.Vec3( 0.5,  0.7, 1.0)
        coords[6] = osg.Vec3(-0.5,  0.7, 1.0)
        coords[7] = osg.Vec3(-0.5, -0.7, 1.0)
        coords[8] = osg.Vec3( 0.0, -0.5, 1.5)
        coords[9] = osg.Vec3( 0.0,  0.5, 1.5)

        for i in range(10):
            coords[i] = coords[i] * scale + offset

        # create geometry
        geometry = osg.Geometry()

        geometry.addPrimitiveSet(primitives)

        geometry.setVertexArray(coords)
        geometry.setVertexIndices(coordIndices)

        geometry.setColorArray(colors)
        geometry.setColorBinding(osg.Geometry.BIND_OVERALL)

        geometry.setNormalArray(normals)
        geometry.setNormalBinding(osg.Geometry.BIND_PER_PRIMITIVE)

        geode = osg.Geode()
        geode.addDrawable(geometry)

        nodes.append(geode)
    print

def LayoutAsGrid():
    print "LayoutAsGrid"
    global Root
    global nodes
    # calculate bounding box
    bbox = osg.BoundingBox()
    for node in nodes:
        bbox.expandBy(node.getBound())

    # setup grid information
    groups = []
    for i in range(GridX * GridY):
        groups.append(osg.Group())

    xGridStart = bbox.xMin()
    xGridSize = (bbox.xMax() - bbox.xMin()) / GridX

    yGridStart = bbox.yMin()
    yGridSize = (bbox.yMax() - bbox.yMin()) / GridY

    # arrange buildings into right grid
    for node in nodes:
        center = node.getBound().center()

        x = int(math.floor((center._x - xGridStart) / xGridSize))
        z = int(math.floor((center._y - yGridStart) / yGridSize))

        groups[z * GridX + x].addChild(node)

    rand = Random().random
    # add nodes to building root
    for g in groups:
        stateset = osg.StateSet()

        material = osg.Material()
        color = osg.Vec4(
            0.5 + rand() / 2.0,
            0.5 + rand() / 2.0,
            0.5 + rand() / 2.0,
            1.0)

        material.setAmbient(osg.Material.FRONT_AND_BACK, color)
        material.setDiffuse(osg.Material.FRONT_AND_BACK, color)
        stateset.setAttributeAndModes(material, osg.StateAttribute.ON)

        g.setStateSet(stateset)

        if UseImpostor:
            impostor = osgSim.Impostor()
            impostor.setImpostorThreshold(Threshold)
            impostor.addChild(g)
            impostor.setRange(0, 0.0, 1.0e7)
            impostor.setCenter(g.getBound().center())
            Root.addChild(impostor)
        else:
            Root.addChild(g)


def main(argv ):
    # use an ArgumentParser object to manage the program arguments.
    arguments = osg.ArgumentParser(argv)

    # set up the usage document, in case we need to print out how to use this program.
    arguments.getApplicationUsage().setDescription(arguments.getApplicationName()+" is the example which demonstrates use impostors - hierachical image cache.")
    arguments.getApplicationUsage().setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...")
    arguments.getApplicationUsage().addCommandLineOption("-h or --help","Display this information")

    # construct the viewer.
    viewer = osgProducer.Viewer(arguments)

    # set up the value with sensible default event handlers.
    viewer.setUpViewer(osgProducer.Viewer.STANDARD_SETTINGS)

    # add local test manipulator more suitable for testing impostors.
    pos = viewer.addCameraManipulator(TestManipulator())
    viewer.selectCameraManipulator(pos)

    # get details on keyboard and mouse bindings used by the viewer.
    viewer.getUsage(arguments.getApplicationUsage())

    # if user request help write it out to cout.
    if arguments.read("-h") or arguments.read("--help"):
        arguments.getApplicationUsage().write(std.out)
        return 1

    # any option left unread are converted into errors to write out later.
    arguments.reportRemainingOptionsAsUnrecognized()

    # report any errors if they have occured when parsing the program aguments.
    if arguments.errors():
        arguments.writeErrorMessages(std.cout)
        return 1


    # load the nodes from the commandline arguments.
    model = osgDB.readNodeFiles(arguments)
    if model:
        # the osgUtil.InsertImpostorsVisitor used lower down to insert impostors
        # only operators on subclass of Group's, if the model top node is not
        # a group then it won't be able to insert an impostor.  We therefore
        # manually insert an impostor above the model.
        if model.asGroup() == None:
            bs = model.getBound()
            if bs.valid():
                impostor = osg.Impostor()

                # standard LOD settings
                impostor.addChild(model)
                impostor.setRange(0,0.0,1.0e7)
                impostor.setCenter(bs.center())

                # impostor specfic settings.
                impostor.setImpostorThresholdToBound(5.0)

                model = impostor

        # we insert an impostor node above the model, so we keep a handle
        # on the rootnode of the model, the is required since the
        # InsertImpostorsVisitor can add a root in automatically and
        # we would know about it, other than by following the parent path
        # up from model.  This is really what should be done, but I'll pass
        # on it right now as it requires a getRoots() method to be added to
        # osg.Node, and we're about to make a release so no features!
        rootnode = osg.Group()
        rootnode.addChild(model)


        # now insert impostors in the model using the InsertImpostorsVisitor.
        ov = osgUtil.InsertImpostorsVisitor()

        # traverse the model and collect all osg.Group's and osg.LOD's.
        # however, don't traverse the rootnode since we want to keep it as
        # the start of traversal, otherwise the insertImpostor could insert
        # and Impostor above the current root, making it nolonger a root!
        model.accept(ov)

        # insert the Impostors above groups and LOD's
        ov.insertImpostors()
    else:
        # no user model so we'll create our own world.
        global Root
        model = Root = osg.Group()
        CreateHouses()
        LayoutAsGrid()

    # add model to viewer.
    viewer.setSceneData(model)

    # register trackball, flight and drive.

    # create the windows and run the threads.
    viewer.realize()

    while not viewer.done():
        # wait for all cull and draw threads to complete.
        viewer.sync()

        # update the scene by traversing it with the the update visitor which will
        # call all node update callbacks and animations.
        viewer.update()

        # fire off the cull and draw traversals of the scene.
        viewer.frame()

    # wait for all cull and draw threads to complete.
    viewer.sync()


    return 0

if __name__ == "__main__":
    main(sys.argv)
