#!/usr/bin/env python

# Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
#
# Permission to copy, use, sell and distribute this software is granted
# provided this copyright notice appears in all copies.
# Permission to modify the code and to distribute modified code is granted
# provided this copyright notice appears in all copies, and a notice
# that the code was modified is included with the copyright notice.
#
# This software is provided "as is" without express or implied warranty,
# and with no claim as to its suitability for any purpose.

# -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2003 Robert Osfield 
#
# This application is open source and may be redistributed and/or modified   
# freely and without restriction, both in commericial and non commericial applications,
# as long as this copyright notice is maintained.
# 
# This application is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#

import sys
from PyOSG import Producer
from PyOSG import osg
from PyOSG import osgDB
from PyOSG import osgUtil
from PyOSG import osgGA
from PyOSG import osgProducer
import OpenGL.GL

keep = []

# XXX Still missing features, doesn't work yet
class OccluderEventHandler(osgGA.GUIEventHandler):
        def __init__(self, viewer):
            osgGA.GUIEventHandler.__init__(self)
            self._viewer = viewer
            self._occluders = None
            self._convexPlanarOccluder = None

        def accept(self, v):
            v.visit(self)

        def handle(self, ea, aa):
            eType = ea.getEventType()
            if eType == osgGA.GUIEventAdapter.KEYDOWN:
                if ea.getKey()==ord('a'):
                    x = ea.getXnormalized()
                    y = ea.getYnormalized()

                    hitList = osgUtil.IntersectVisitor.HitList()
                    if not self._viewer.computeIntersections(x, y, hitList):
                        return True

                    if not hitList.empty():
                        hit = hitList.front()
                        self.addPoint(hit.getWorldIntersectPoint())
                    return True
                elif ea.getKey()==ord('e'):
                    self.endOccluder()
                    return True
                elif ea.getKey()==ord('O'):
                    if self._occluders:
                        osgDB.writeNodeFile(self._occluders,"saved_occluders.osg")
                        print "saved occluders to 'saved_occluders.osg'"
                    else:
                        print "no occluders to save"
                    return True
                return False
            return False

        def addPoint(self, pos):
            print "add point "
            
            if not self._convexPlanarOccluder:
                self._convexPlanarOccluder = osg.ConvexPlanarOccluder()
            
            occluder = self._convexPlanarOccluder.getOccluder()
            occluder.add(pos)

        def endOccluder(self):
            if self._convexPlanarOccluder:
                if len(self._convexPlanarOccluder.getOccluder().getVertexList()) >=3:
                    occluderNode = osg.OccluderNode()
                    occluderNode.setOccluder(self._convexPlanarOccluder)

                    if not self._occluders:
                        self._occluders = osg.Group()
                        if self.rootNode():
                            self.rootNode().addChild(self._occluders)
                    self._occluders.addChild(occluderNode)

                    print "created occluder"
                else:
                    print "Occluder requires at least 3 points to create occluder."
            else:
                print "No occluder points to create occluder with."
            
            # reset current occluder.
            self._convexPlanarOccluder = None

        def rootNode(self):
            return self._viewer.getSceneData().asGroup()

def createOccluder(v1, v2, v3, v4, holeRatio=-1.0):
    # create and occluder which will site along side the loadmodel model.
    occluderNode = osg.OccluderNode()

    # create the convex planer occluder 
    cpo = osg.ConvexPlanarOccluder()

    # attach it to the occluder node.
    occluderNode.setOccluder(cpo)
    occluderNode.setName("occluder")
    
    # set the occluder up for the front face of the bounding box.
    occluder = cpo.getOccluder()
    occluder.add(v1)
    occluder.add(v2)
    occluder.add(v3)
    occluder.add(v4)

    # create a whole at the center of the occluder if needed.
    if holeRatio>0.0:
        # create hole.
        ratio = holeRatio
        one_minus_ratio = 1-ratio
        center = (v1+v2+v3+v4)*0.25
        v1dash = v1*ratio + center*one_minus_ratio
        v2dash = v2*ratio + center*one_minus_ratio
        v3dash = v3*ratio + center*one_minus_ratio
        v4dash = v4*ratio + center*one_minus_ratio

        hole = osg.ConvexPlanarPolygon ()
        hole.add(v1dash)
        hole.add(v2dash)
        hole.add(v3dash)
        hole.add(v4dash)

        cpo.addHole(hole)
    

    # create a drawable for occluder.
    geom = osg.Geometry()
    
    # getVertexList returns a tuple containing vertices
    # it can be fed straight into setVertexArray
    vlist = occluder.getVertexList()
    # coords = osg.Vec3Array()
    # coords.set(vlist)
    geom.setVertexArray(vlist)
    
    colors = osg.Vec4Array(1)
    colors[0].set(1.0,1.0,1.0,0.5)
    geom.setColorArray(colors)
    geom.setColorBinding(osg.Geometry.BIND_OVERALL)
    
    geom.addPrimitiveSet(osg.DrawArrays(osg.PrimitiveSet.QUADS,0,4))
    
    geode = osg.Geode()
    geode.addDrawable(geom)
    
    stateset = osg.StateSet()
    stateset.setMode(OpenGL.GL.GL_LIGHTING,osg.StateAttribute.OFF)
    stateset.setMode(OpenGL.GL.GL_BLEND,osg.StateAttribute.ON)
    stateset.setRenderingHint(osg.StateSet.TRANSPARENT_BIN)

    geom.setStateSet(stateset)

    # add the occluder geode as a child of the occluder,
    # as the occluder can't self occlude its subgraph the
    # geode will never be occluder by this occluder.
    occluderNode.addChild(geode);    

    return occluderNode

def createOccludersAroundModel(model):
    scene = osg.Group()
    scene.setName("rootgroup")

    # add the loaded model into a the scene group.
    scene.addChild(model)
    model.setName("model")

    # get the bounding volume of the model.
    bs = model.getBound()
    
    # create a bounding box around the sphere.
    bb = osg.BoundingBox()
    bb.expandBy(bs)

    # front
    scene.addChild(createOccluder(bb.corner(0),
                                  bb.corner(1),
                                  bb.corner(5),
                                  bb.corner(4)))

    # right side
    scene.addChild(createOccluder(bb.corner(1),
                                  bb.corner(3),
                                  bb.corner(7),
                                  bb.corner(5)))

    # left side
    scene.addChild(createOccluder(bb.corner(2),
                                  bb.corner(0),
                                  bb.corner(4),
                                  bb.corner(6)))

    # back side
    scene.addChild(createOccluder(bb.corner(3),
                                  bb.corner(2),
                                  bb.corner(6),
                                  bb.corner(7),
                                  0.5)); # create a hole half the size of the occluder.
    return scene

def main(argv):
    # use an ArgumentParser object to manage the program arguments.
    arguments =  osg.ArgumentParser(argv)
    
    # set up the usage document, in case we need to print out how to use this program.
    arguments.getApplicationUsage().setDescription(arguments.getApplicationName()+" is the example which demonstrates use of convex planer occluders.");
    arguments.getApplicationUsage().setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...")
    arguments.getApplicationUsage().addCommandLineOption("-h or --help","Display this information")
    arguments.getApplicationUsage().addCommandLineOption("-m","Mannually create occluders");
    

    # construct the viewer.
    viewer = osgProducer.Viewer(arguments)

    # set up the value with sensible default event handlers.
    viewer.setUpViewer(osgProducer.Viewer.STANDARD_SETTINGS)

    manuallyCreateOccluders = False
    while arguments.read("-m"):
        manuallyCreateOccluders = True


    if manuallyCreateOccluders:
        oeh = OccluderEventHandler(viewer)
        global keep
        keep.append(oeh)
        viewer.eventHandler_push_front(oeh)

    # get details on keyboard and mouse bindings used by the viewer.
    viewer.getUsage(arguments.getApplicationUsage())

    # if user request help write it out to cout.
    if arguments.read("-h") or arguments.read("--help"):
        arguments.getApplicationUsage().write()
        return 1

    # any option left unread are converted into errors to write out later.
    arguments.reportRemainingOptionsAsUnrecognized()

    # report any errors if they have occured when parsing the program aguments.
    if arguments.errors():
        arguments.writeErrorMessages()
        return 1

    loadedmodel = osgDB.readNodeFiles(arguments)
    if not loadedmodel:
        return 1
    
    # run optimization over the scene graph
    optimzer = osgUtil.Optimizer()
    optimzer.optimize(loadedmodel)

    # add the occluders to the loaded model.
    rootnode = None
    
    if manuallyCreateOccluders:
        rootnode = osg.Group()
        rootnode.addChild(loadedmodel)
    else :
        rootnode = createOccludersAroundModel(loadedmodel)
    
    # add model to viewer
    viewer.setSceneData(rootnode)

    # create the windows and run the threads.
    viewer.realize()

    while not viewer.done():
        # wait for all cull and draw threads to complete.
        viewer.sync()

        # update the scene by traversing it with the the update visitor which will
        # call all node update callbacks and animations.
        viewer.update()
         
        # fire off the cull and draw traversals of the scene.
        viewer.frame()
    
    # wait for all cull and draw threads to complete before exit.
    viewer.sync()

    return 0

if __name__ == "__main__":
    main(sys.argv)
