// Copyright (C) 2002-2005 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/scope.hpp>

#include <osg/BlendColor>

#include "StateAttribute.hpp"
#include "held_ptr.hpp"

using namespace boost::python;

namespace {
DEFINE_STATEATTRIBUTE_CAST(BlendColor)
}

namespace PyOSG {
void init_BlendColor()
{
    REGISTER_STATEATTRIBUTE_CAST(BlendColor);

    class_<osg::BlendColor, osg::ref_ptr<osg::BlendColor>, bases<osg::StateAttribute> > blend_color("BlendColor", 
            "Encapsulates OpenGL blend/transparency state.\n",
            no_init);
    scope blend_color_scope(blend_color);
    blend_color.def(init<>())
        .def("setConstantColor", &osg::BlendColor::setConstantColor)
        .def("getConstantColor", &osg::BlendColor::getConstantColor)
        ;
}
}
