// Copyright (C) 2005 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/call_method.hpp>
#include <boost/python/object.hpp>
#include <boost/python/extract.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/python/copy_const_reference.hpp>

#include <osg/NodeCallback>
#include <osg/NodeVisitor>
#include <osg/ClusterCullingCallback>

#include "held_ptr.hpp"
#include "lock.hpp"

using namespace boost::python;

typedef osg::ClusterCullingCallback ClusterCullingCallback;

namespace PyOSG {
class PyClusterCullingCallback : public ClusterCullingCallback
{
  public:
    PyClusterCullingCallback(PyObject * p) : ClusterCullingCallback(), _self(p) {}
   ~PyClusterCullingCallback() {}

    virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
    {
        // std::cerr << "PyClusterCullingCallback::operator(), thread id = " << PyThread_get_thread_ident() << std::endl;
        PYOSG_LOCK_INTERPRETER();
        try {
            call_method<void>(_self, "apply", ptr(node), ptr(nv));
        } catch(...) {
            handle_exception();
            PyErr_Print();
        }
        PYOSG_RELEASE_INTERPRETER();
    }
    void traverse_imp(osg::Node* node,osg::NodeVisitor* nv)
    {
        this->ClusterCullingCallback::traverse(node, nv);
    }
  private:
      PyObject * _self;
};

void init_ClusterCullingCallback()
{
    class_<osg::ClusterCullingCallback, osg::ref_ptr<PyClusterCullingCallback>, bases<osg::Drawable::CullCallback, osg::NodeCallback>, boost::noncopyable>("ClusterCullingCallback", no_init)
        .def(init<>())
        .def("traverse", &PyClusterCullingCallback::traverse_imp)

        .def("computeFrom", &PyClusterCullingCallback::computeFrom)
        .def("set", &PyClusterCullingCallback::set)


        .def("setControlPoint",
                &PyClusterCullingCallback::setControlPoint)
//        .def("getControlPoint",
//                (const osg::Vec3& (osg::ClusterCullingCallback::*)() const)
//                &PyClusterCullingCallback::getControlPoint,
//                return_value_policy<copy_const_reference>())

        .def("setNormal", &PyClusterCullingCallback::setNormal)
//        .def("getNormal", &PyClusterCullingCallback::getNormal,
//                (const osg::Vec3& (osg::ClusterCullingCallback::*)() const)
//                &PyClusterCullingCallback::getNormal,
//                return_value_policy<copy_const_reference>())

        .def("setRadius", &PyClusterCullingCallback::setRadius)
        .def("getRadius", &PyClusterCullingCallback::getRadius)

        .def("setDeviation", &PyClusterCullingCallback::setDeviation)
        .def("getDeviation", &PyClusterCullingCallback::getDeviation)
        ;
}
}
