// Copyright (C) 2002-2005 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/python/copy_non_const_reference.hpp>

#include <osg/ColorMatrix>

#include "StateAttribute.hpp"
#include "held_ptr.hpp"

using namespace boost::python;

namespace {
osg::ColorMatrix * asColorMatrix(osg::StateAttribute * self) { return dynamic_cast<osg::ColorMatrix*>(self); }
}

namespace PyOSG {
void init_ColorMatrix()
{
    register_castStateAttribute("asColorMatrix", &asColorMatrix);

    class_<osg::ColorMatrix, osg::ref_ptr<osg::ColorMatrix>, bases<osg::StateAttribute> >("ColorMatrix", 
        "Texture Matrix state class for encapsulating OpenGL texture matrix functionality.\n",
        no_init)
        .def(init<>())
        .def("setMatrix",
            &osg::ColorMatrix::setMatrix,
            "Set the color matrix")
        .def("getMatrix", 
            (osg::Matrix &(osg::ColorMatrix::*)())&osg::ColorMatrix::getMatrix,
            return_value_policy<copy_non_const_reference>(),
            "Get the color matrix\n")
        ;
}
} // namespace PyOSG
