// Copyright (C) 2002-2005 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/implicit.hpp>
#include <boost/python/scope.hpp>
#include <boost/python/enum.hpp>

#include <osg/CullFace>

#include "StateAttribute.hpp"
#include "held_ptr.hpp"

using namespace boost::python;

namespace {
osg::CullFace * asCullFace(osg::StateAttribute * self) { return dynamic_cast<osg::CullFace*>(self); }
}

namespace PyOSG {
void init_CullFace()
{
    register_castStateAttribute("asCullFace", &asCullFace);

    class_<osg::CullFace, osg::ref_ptr<osg::CullFace>, bases<osg::StateAttribute> > cullFace("CullFace", 
            "Class to globally enable/disable OpenGL's polygon culling mode\n", no_init);

    scope cullFace_scope(cullFace);

    cullFace
        .def(init<>())

        .def("setMode", &osg::CullFace::setMode)
        .def("getMode", &osg::CullFace::getMode)
        ;

#   define OSG_ENUM_MODE(VALUE) \
    (mode.value(#VALUE, osg::CullFace::VALUE), \
    cullFace.def(#VALUE, object(osg::CullFace::VALUE)))
    enum_<osg::CullFace::Mode> mode("Mode");

    OSG_ENUM_MODE(FRONT);
    OSG_ENUM_MODE(BACK);
    OSG_ENUM_MODE(FRONT_AND_BACK);
}
} // namespace PyOSG
