// Copyright (C) 2002-2005 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/copy_non_const_reference.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/ref.hpp>

#include <osg/DrawPixels>

#include <string>
#include <iostream>

#include "held_ptr.hpp"
#include "Math.hpp"

using namespace boost::python;
using namespace PyOSG;

namespace PyOSG {
void init_DrawPixels()
{
    class_<osg::DrawPixels, osg::ref_ptr<osg::DrawPixels>, bases<osg::Drawable> >("DrawPixels",
            "DrawPixels is an osg::Drawable subclass which encapsulates the drawing of\n"
            "images using glDrawPixels.\n",
            no_init)
        .def(init<>())
        .def("setPosition", &osg::DrawPixels::setPosition)
        .def("getPosition", 
            (osg::Vec3& (osg::DrawPixels::*)()) 
            &osg::DrawPixels::getPosition,
            return_value_policy<copy_non_const_reference>())

        .def("setImage", &osg::DrawPixels::setImage)
        .def("getImage", 
            (osg::Image* (osg::DrawPixels::*)())
            &osg::DrawPixels::getImage,
            return_value_policy<manage_osg_object>())

        .def("setUseSubImage", &osg::DrawPixels::setUseSubImage)
        .def("getUseSubImage", &osg::DrawPixels::getUseSubImage)

        .def("setSubImageDimensions", &osg::DrawPixels::setSubImageDimensions)
        .def("getSubImageDimensions", &osg::DrawPixels::getSubImageDimensions)
        ;
}
} // namespace PyOSG
