// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/scope.hpp>
#include <boost/python/enum.hpp>

#include <osg/FrontFace>

#include "StateAttribute.hpp"
#include "held_ptr.hpp"

using namespace boost::python;

namespace {
osg::FrontFace * asFrontFace(osg::StateAttribute * self) { return dynamic_cast<osg::FrontFace*>(self); }
}

namespace PyOSG {
void init_FrontFace()
{
    register_castStateAttribute("asFrontFace", &asFrontFace);

    class_<osg::FrontFace, osg::ref_ptr<osg::FrontFace>, bases<osg::StateAttribute> > frontFace("FrontFace", no_init);
    
    scope frontFace_scope(frontFace);
    frontFace
        .def(init<>())

        .def("setMode",
            &osg::FrontFace::setMode)
        .def("getMode",
            &osg::FrontFace::getMode)
        ;
#   define OSG_ENUM_MODE(VALUE) \
    (mode.value(#VALUE, osg::FrontFace::VALUE), \
    frontFace.def(#VALUE, object(osg::FrontFace::VALUE)))
    enum_<osg::FrontFace::Mode> mode("Mode");

    OSG_ENUM_MODE(CLOCKWISE);
    OSG_ENUM_MODE(COUNTER_CLOCKWISE);
}
} // namespace PyOSG
