// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/python/copy_const_reference.hpp>
#include <boost/ref.hpp>

#include <osg/MatrixTransform>

#include <string>
#include <iostream>

#include "Transform.hpp"
#include "held_ptr.hpp"

using namespace boost::python;

namespace {
DEFINE_TRANSFORM_CAST(MatrixTransform)
}

namespace PyOSG {
void init_MatrixTransform()
{
    REGISTER_TRANSFORM_CAST(MatrixTransform)

    class_<osg::MatrixTransform, osg::ref_ptr<osg::MatrixTransform>, bases<osg::Transform>, boost::noncopyable > mtransform("MatrixTransform", 
            "MatrixTransform - is a subclass of Transform which has an osg::Matrix\n"
            "which represent a 4x4 transformation of its children from local cordinates\n"
            "into the Transform's parent coordinates.\n",
            no_init);

    mtransform
        .def(init<>())
        .def(init<osg::Matrix&>())
        .def("setMatrix",
            &osg::MatrixTransform::setMatrix)
        .def("getMatrix",
            &osg::MatrixTransform::getMatrix,
            return_value_policy<copy_const_reference>())
        .def("preMult",
            &osg::MatrixTransform::preMult)
        .def("postMult",
            &osg::MatrixTransform::postMult)
        .def("getInverseMatrix",
            &osg::MatrixTransform::getInverseMatrix,
            return_value_policy<copy_const_reference>())
        .def("computeLocalToWorldMatrix",
            &osg::MatrixTransform::computeLocalToWorldMatrix)
        .def("computeWorldToLocalMatrix",
            &osg::MatrixTransform::computeWorldToLocalMatrix)
         ;
}
}
