// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/return_internal_reference.hpp>

#include <osg/OccluderNode>

#include <string>
#include <iostream>

#include "Node.hpp"
#include "held_ptr.hpp"

using namespace boost::python;

namespace {
DEFINE_NODE_CAST(OccluderNode)
}

namespace PyOSG {
void init_OccluderNode()
{
    REGISTER_NODE_CAST(OccluderNode)

    class_<osg::OccluderNode, osg::ref_ptr<osg::OccluderNode>, bases<osg::Group>, boost::noncopyable >("OccluderNode",
        "OccluderNode is a Group node which allows OccluderNodeing between children."
        "Typical uses would be for objects which might need to be rendered"
        "differently at different times, for instance a OccluderNode could be used "
        "to represent the different states of a traffic light.",
        no_init
        )
    .def(init<>())

    .def("setOccluder", &osg::OccluderNode::setOccluder,
        "Attach a ConvexPlanarOccluder to an OccluderNode."
        )
    
    .def("getOccluder", (osg::ConvexPlanarOccluder* (osg::OccluderNode::*)()) &osg::OccluderNode::getOccluder,
        return_internal_reference<>(),
        "Get the ConvexPlanarOccluder* attached to a OccluderNode."
        )
     ;
}
}
