// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/python/copy_const_reference.hpp>

#include <osg/Point>

#include "StateAttribute.hpp"
#include "held_ptr.hpp"

using namespace boost::python;

namespace {
osg::Point * asPoint(osg::StateAttribute * self) { return dynamic_cast<osg::Point*>(self); }
}

namespace PyOSG {
void init_Point()
{
    register_castStateAttribute("asPoint", &asPoint);

    class_<osg::Point, osg::ref_ptr<osg::Point>, bases<osg::StateAttribute> >("Point", no_init)
         .def(init<>())
         .def("setSize", &osg::Point::setSize)
         .def("getSize", &osg::Point::getSize)
         .def("setFadeThresholdSize", &osg::Point::setFadeThresholdSize)
         .def("getFadeThresholdSize", &osg::Point::getFadeThresholdSize)
         .def("setDistanceAttenuation", &osg::Point::setDistanceAttenuation)
         .def("getDistanceAttenuation", &osg::Point::getDistanceAttenuation, return_value_policy<copy_const_reference>())
         .def("setMinSize", &osg::Point::setMinSize)
         .def("setMinSize", &osg::Point::setMinSize)
         .def("getMaxSize", &osg::Point::getMaxSize)
         .def("getMaxSize", &osg::Point::getMaxSize)
         ;
}
}
