// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/python/copy_const_reference.hpp>

#include <osg/Projection>

#include "Node.hpp"
#include "held_ptr.hpp"

using namespace boost::python;

namespace {
DEFINE_NODE_CAST(Projection)
}

namespace PyOSG {
void init_Projection()
{
    REGISTER_NODE_CAST(Projection)

    class_<osg::Projection, osg::ref_ptr<osg::Projection>, bases<osg::Group> >("Projection", 
            "Projection nodes set up the frustum/orthographic projection used when rendering the scene .",
            no_init)
        .def(init<>())
        .def(init<osg::Matrix &>())
        .def("setMatrix", &osg::Projection::setMatrix)
        .def("getMatrix", &osg::Projection::getMatrix, return_value_policy<copy_const_reference>())
        .def("preMult", &osg::Projection::preMult)
        .def("postMult", &osg::Projection::postMult)
        ;
}
} // namespace PyOSG
