// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/enum.hpp>
#include <boost/python/scope.hpp>

#include <osg/ShadeModel>

#include "StateAttribute.hpp"
#include "held_ptr.hpp"

using namespace boost::python;

namespace {
void osgShadeModel_setMode(osg::ShadeModel * self, int mode)
{
    self->setMode(osg::ShadeModel::Mode(mode));
}

osg::ShadeModel * asShadeModel(osg::StateAttribute * self) { return dynamic_cast<osg::ShadeModel*>(self); }

} // namespace

namespace PyOSG {
void init_ShadeModel()
{
    register_castStateAttribute("asShadeModel", &asShadeModel);

    class_<osg::ShadeModel, osg::ref_ptr<osg::ShadeModel>, bases<osg::StateAttribute> > shade("ShadeModel", no_init);

    scope shade_scope(shade);
    shade.def(init<>())

        .def("setMode", &osg::ShadeModel::setMode)
        .def("getMode", &osg::ShadeModel::getMode);

#   define OSG_ENUM_MODE(VALUE) \
    (mode.value(#VALUE, osg::ShadeModel::VALUE), \
    shade.def(#VALUE, object(osg::ShadeModel::VALUE)))
    enum_<osg::ShadeModel::Mode> mode("Mode");
    OSG_ENUM_MODE(FLAT);
    OSG_ENUM_MODE(SMOOTH);
}
} // namespace PyOSG


