// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/tuple.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/python/copy_const_reference.hpp>
#include <boost/python/extract.hpp>
#include <boost/python/detail/api_placeholder.hpp>
#include <boost/python/scope.hpp>
#include <boost/python/enum.hpp>

#include <string>
#include <iostream>
#include <sstream>

#include <osg/Shape>

#include "held_ptr.hpp"
#include "Math.hpp"

using namespace boost::python;


namespace PyOSG {
extern void init_Shape2();
extern void init_Shape3();

void init_Shape()
{
    class_<osg::Shape, osg::ref_ptr<osg::Shape>, bases<osg::Object>, boost::noncopyable> shape("Shape", no_init);

    // Sphere

    class_<osg::Sphere, osg::ref_ptr<osg::Sphere>, bases<osg::Shape>, boost::noncopyable> sphere("Sphere", no_init);
    sphere.def(init<>());
    sphere.def(init<const osg::Vec3&, float>());
    sphere.def("valid", &osg::Sphere::valid);

    sphere.def("set", &osg::Sphere::set);

    sphere.def("setCenter", &osg::Sphere::setCenter);
    sphere.def("getCenter", &osg::Sphere::getCenter, return_value_policy<copy_const_reference>());

    sphere.def("setRadius", &osg::Sphere::setRadius);
    sphere.def("getRadius", &osg::Sphere::getRadius);

    // Box

    class_<osg::Box, osg::ref_ptr<osg::Box>, bases<osg::Shape>, boost::noncopyable> box("Box", no_init);
    box.def(init<>());
    box.def(init<const osg::Vec3&, float>());
    box.def(init<const osg::Vec3&, float, float, float>());
    box.def("valid", &osg::Box::valid);

    box.def("set", &osg::Box::set);

    box.def("setCenter", &osg::Box::setCenter);
    box.def("getCenter", &osg::Box::getCenter, return_value_policy<copy_const_reference>());

    box.def("setHalfLengths", &osg::Box::setHalfLengths);
    box.def("getHalfLengths", &osg::Box::getHalfLengths, return_value_policy<copy_const_reference>());

    box.def("setRotation", &osg::Box::setRotation);
    box.def("getRotation", &osg::Box::getRotation, return_value_policy<copy_const_reference>());
    box.def("computeRotationMatrix", &osg::Box::computeRotationMatrix);
    box.def("zeroRotation", &osg::Box::zeroRotation);

    init_Shape2();
    init_Shape3();
}
}

