// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/python/copy_const_reference.hpp>

#include <osg/Shape>

#include "held_ptr.hpp"
#include "Math.hpp"

using namespace boost::python;

namespace PyOSG {
void init_Shape3()
{
    // InfinitePlane

    //  class_<osg::InfinitePlane, osg::ref_ptr<osg::InfinitePlane>, bases<osg::Shape>, boost::noncopyable> infplane("InfinitePlane");

    // TriangleMesh

    class_<osg::TriangleMesh, osg::ref_ptr<osg::TriangleMesh>, bases<osg::Shape>, boost::noncopyable> trimesh("TriangleMesh", no_init);
    trimesh.def(init<>());
    trimesh.def("setVertices", &osg::TriangleMesh::setVertices);
    trimesh.def("getVertices",
        (osg::Vec3Array *(osg::TriangleMesh::*)()) &osg::TriangleMesh::getVertices,
        return_value_policy<manage_osg_object>());

    trimesh.def("setIndices", &osg::TriangleMesh::setIndices);
    trimesh.def("getIndices",
        (osg::IndexArray *(osg::TriangleMesh::*)()) &osg::TriangleMesh::getIndices,
        return_value_policy<manage_osg_object>());

    // ConvexHull

    class_<osg::ConvexHull, osg::ref_ptr<osg::ConvexHull>, bases<osg::TriangleMesh>, boost::noncopyable> convhull("ConvexHull", no_init);
    convhull.def(init<>());

    // HeightField - abstract base class

    class_<osg::HeightField, osg::ref_ptr<osg::HeightField>, bases<osg::Shape>, boost::noncopyable> hfield("HeightField", no_init);
    hfield.def(init<>());
    hfield.def("allocate", &osg::HeightField::allocate);
    hfield.def("getNumColumns", &osg::HeightField::getNumColumns);
    hfield.def("getNumRows", &osg::HeightField::getNumRows);

    hfield.def("setOrigin", &osg::HeightField::setOrigin);
    hfield.def("getOrigin", &osg::HeightField::getOrigin, return_value_policy<copy_const_reference>());

    hfield.def("setXInterval", &osg::HeightField::setXInterval);
    hfield.def("getXInterval", &osg::HeightField::getXInterval);

    hfield.def("setYInterval", &osg::HeightField::setYInterval);
    hfield.def("getYInterval", &osg::HeightField::getYInterval);

    hfield.def("setSkirtHeight", &osg::HeightField::setSkirtHeight);
    hfield.def("getSkirtHeight", &osg::HeightField::getSkirtHeight);

    hfield.def("setBorderWidth", &osg::HeightField::setBorderWidth);
    hfield.def("getBorderWidth", &osg::HeightField::getBorderWidth);

    hfield.def("setRotation", &osg::HeightField::setRotation);
    hfield.def("getRotation", &osg::HeightField::getRotation, return_value_policy<copy_const_reference>());
    hfield.def("computeRotationMatrix", &osg::HeightField::computeRotationMatrix);
    hfield.def("zeroRotation", &osg::HeightField::zeroRotation);

    hfield.def("setHeight", &osg::HeightField::setHeight);
    hfield.def("getHeight",
        (float (osg::HeightField::*)(unsigned int, unsigned int) const)
        &osg::HeightField::getHeight);

    hfield.def("getVertex", &osg::HeightField::getVertex);
    hfield.def("getNormal", &osg::HeightField::getNormal);

    // Grid

    // class_<osg::Grid, osg::ref_ptr<osg::Grid>, bases<osg::HeightField>, boost::noncopyable> grid("Grid", no_init);
    // grid.def(init<>());
    // grid.def("allocateGrid", &osg::Grid::allocateGrid);
//    grid.def("getHeight", (float (osg::Grid::*)(unsigned int, unsigned int) const) &osg::Grid::getHeight);


    // CompositeShape

    class_<osg::CompositeShape, osg::ref_ptr<osg::CompositeShape>, bases<osg::Shape>, boost::noncopyable> cshape("CompositeShape", no_init);
    cshape.def(init<>());
    cshape.def("setShape", &osg::CompositeShape::setShape,
        "Set the shape that encloses all of the children.");

    cshape.def("getShape", (osg::Shape *(osg::CompositeShape::*)()) &osg::CompositeShape::getShape,
        "Get the shape that encloses all of the children.",
        return_value_policy<manage_osg_object>());

    cshape.def("getNumChildren", &osg::CompositeShape::getNumChildren,
        "Get the number of children of this composite shape.");

    cshape.def("getChild", (osg::Shape *(osg::CompositeShape::*)(unsigned int)) &osg::CompositeShape::getChild,
        "Get a child",
        return_value_policy<manage_osg_object>());

    cshape.def("addChild", &osg::CompositeShape::addChild,
        "Add a child to the list");

    cshape.def("removeChild", &osg::CompositeShape::removeChild,
        "remove a child from the list.");

    cshape.def("findChildNo", &osg::CompositeShape::findChildNo,
        "find the index number of child, if child is not found then it returns getNumChildren(),\n"
        "so should be used in similar sytle of STL's result!=end().");
}
}
