// Copyright (C) 2005 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/python/scope.hpp>
#include <boost/ref.hpp>

#include <osg/TexGenNode>

#include <string>
#include <iostream>

#include "Group.hpp"
#include "held_ptr.hpp"

using namespace osg;
using namespace boost::python;

namespace {
DEFINE_GROUP_CAST(TexGenNode)
}

namespace PyOSG {
void init_TexGenNode()
{
    REGISTER_GROUP_CAST(TexGenNode)

    class_<TexGenNode, ref_ptr<TexGenNode>, bases<Group> > texgennode("TexGenNode",
        "Node for defining the position of TexGen in the scene.\n",
        no_init);

    scope texgennode_scope(texgennode);

    texgennode.def(init<>())
        .def("setTextureUnit",
            &TexGenNode::setTextureUnit)
        .def("getTextureUnit",
            &TexGenNode::getTextureUnit)

        .def("setTexGen",
            &TexGenNode::setTexGen)
        .def("getTexGen",
            (osg::TexGen*  (TexGenNode::*)()) &TexGenNode::getTexGen,
            return_value_policy<manage_osg_object>())
         ;
}
}
