// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/return_internal_reference.hpp>

#include <osg/TexMat>

#include "StateAttribute.hpp"
#include "held_ptr.hpp"

using namespace boost::python;

namespace {
osg::TexMat * asTexMat(osg::StateAttribute * self) { return dynamic_cast<osg::TexMat*>(self); }
}

namespace PyOSG {
void init_TexMat()
{
    register_castStateAttribute("asTexMat", &asTexMat);

    class_<osg::TexMat, osg::ref_ptr<osg::TexMat>, bases<osg::StateAttribute> >("TexMat",
            "Texture Matrix state class for encapsulating OpenGL texture matrix functionality.",
            no_init)
        .def(init<>())
        .def("setMatrix", &osg::TexMat::setMatrix)
        .def("getMatrix", (osg::Matrix&(osg::TexMat::*)())&osg::TexMat::getMatrix, return_internal_reference<>())
        ;
}
}
