// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
//

#include <boost/python/class.hpp>
#include <boost/python/return_internal_reference.hpp>

#include <osgParticle/AccelOperator>

#include "held_ptr.hpp"

using namespace boost::python;

namespace PyOSGParticle {
void init_AccelOperator()
{
    class_<osgParticle::AccelOperator, osg::ref_ptr<osgParticle::AccelOperator>, bases<osgParticle::Operator>, boost::noncopyable > operat("AccelOperator",
        "An operator class that applies a constant acceleration to the particles.",
        no_init);

    operat
        .def(init<>())

        .def("getAcceleration", 
            &osgParticle::AccelOperator::getAcceleration,
            "Get the acceleration vector.",
            return_internal_reference<>())

        .def("setAcceleration", 
            &osgParticle::AccelOperator::setAcceleration,
            "Set the acceleration vector.")

        .def("setToGravity", 
            &osgParticle::AccelOperator::setToGravity,
            "Quickly set the acceleration vector to the gravity on earth (0, 0, -9.81)."
            "The acceleration will be multiplied by the <CODE>scale</CODE> parameter.")
        ;
}
}
