// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
//

#include <boost/python/class.hpp>
#include <boost/python/scope.hpp>
#include <boost/python/enum.hpp>
#include <boost/python/return_internal_reference.hpp>

#include <osgParticle/CenteredPlacer>

#include "held_ptr.hpp"

using namespace boost::python;

namespace PyOSGParticle {
void init_CenteredPlacer()
{
    class_<osgParticle::CenteredPlacer, osg::ref_ptr<osgParticle::CenteredPlacer>, bases<osgParticle::Placer>, boost::noncopyable > placer("CenteredPlacer",
        "An abstract placer base class for placers which need a <I>center point</I>.",
        no_init);

    placer
        .def("getCenter", 
            &osgParticle::CenteredPlacer::getCenter,
            "Get the center point.",
            return_internal_reference<>())

        .def("setCenter", 
            (void (osgParticle::CenteredPlacer::*)(const osg::Vec3&))
            &osgParticle::CenteredPlacer::setCenter,
            "Set the center point.")

        .def("setCenter",
            (void (osgParticle::CenteredPlacer::*)(float, float, float)) 
            &osgParticle::CenteredPlacer::setCenter,
            "Set the center point.")
        ;
}
}
