// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
//

#include <boost/python/class.hpp>
#include <boost/python/return_internal_reference.hpp>

#include <osgParticle/Emitter>

#include "held_ptr.hpp"

using namespace boost::python;

namespace PyOSGParticle {
void init_Emitter()
{
    class_<osgParticle::Emitter, osg::ref_ptr<osgParticle::Emitter>, bases<osgParticle::ParticleProcessor>, boost::noncopyable > emitter("Emitter",
        "An abstract base class for particle emitters."
        "Descendant classes must override the <CODE>emit()</CODE> method to generate new particles by"
        "calling the <CODE>ParticleSystem::createParticle()</CODE> method on the particle system associated"
        "to the emitter.",
        no_init);

    emitter
        .def("getParticleTemplate", 
            &osgParticle::Emitter::getParticleTemplate,
            "Get the particle template.",
            return_internal_reference<>())

        .def("setParticleTemplate", 
            &osgParticle::Emitter::setParticleTemplate,
            "Set the particle template (particle is copied).")

        .def("getUseDefaultTemplate", 
            &osgParticle::Emitter::getUseDefaultTemplate,
            "Return whether the particle system's default template should be used.")

        .def("setUseDefaultTemplate", 
            &osgParticle::Emitter::setUseDefaultTemplate,
            "Set whether the default particle template should be used."
            "When this flag is true, the particle template is ignored, and the"
            "particle system's default template is used instead.")
        ;
}
}
