// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
//

#include <boost/python/class.hpp>
#include <boost/python/scope.hpp>
#include <boost/python/enum.hpp>
#include <boost/python/return_internal_reference.hpp>

#include <osgParticle/ForceOperator>

#include "held_ptr.hpp"

using namespace boost::python;

namespace PyOSGParticle {
void init_ForceOperator()
{
    class_<osgParticle::ForceOperator, osg::ref_ptr<osgParticle::ForceOperator>, bases<osgParticle::Operator>, boost::noncopyable > operat("ForceOperator",
        "An operator that applies a constant force to the particles."
        "Remember that if the mass of particles is expressed in <U>kg</U> and the lengths are"
        "expressed in <U>meters</U>, then the force should be expressed in <U>Newtons</U>.",
        no_init);

    operat.def(init<>());

    operat.def("getForce", &osgParticle::ForceOperator::getForce,
        "Get the force vector.",
        return_internal_reference<>());

    operat.def("setForce", &osgParticle::ForceOperator::setForce,
        "Set the force vector.");
}
}
