// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
//

#include <boost/python/class.hpp>
#include <boost/python/scope.hpp>
#include <boost/python/enum.hpp>

#include <osgParticle/Particle>
#include <osgParticle/Placer>

#include "held_ptr.hpp"

using namespace boost::python;

namespace PyOSGParticle {
void init_Placer()
{
    class_<osgParticle::Placer, osg::ref_ptr<osgParticle::Placer>, bases<osg::Object>, boost::noncopyable > placer("Placer",
        "An abstract base class for implementing <I>particle placers</I>. A placer is an object which take"
        "a particle as input, and places it somewhere by setting its position vector. Placer objects are"
        "used by the <CODE>ModularEmitter</CODE> class as part of the particle emission process.",
        no_init);

    placer.def("place", &osgParticle::Placer::place,
        "Place a particle. Must be implemented in descendant classes.");
}
}
