// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
//

#include <boost/python/class.hpp>
#include <boost/python/scope.hpp>
#include <boost/python/enum.hpp>

#include <osgParticle/Particle>
#include <osgParticle/Shooter>

#include "held_ptr.hpp"

using namespace boost::python;
using namespace osgParticle;

namespace PyOSGParticle {
void init_Shooter()
{
    class_<osgParticle::Shooter, osg::ref_ptr<osgParticle::Shooter>, bases<osg::Object>, boost::noncopyable > shooter("Shooter",
        "An abstract base class used by ModularEmitter to \"shoot\" the particles after they have been placed. "
        "Descendants of this class must override the <CODE>shoot()</CODE> method.",
        no_init);

    shooter.def("shoot", &osgParticle::Shooter::shoot,
            "Shoot a particle. Must be overriden by descendants."
            "This method should only set the velocity vector of particle <CODE>P</CODE>, leaving other"
            "attributes unchanged.");
}
}
