// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
//

#include <boost/python/class.hpp>
#include <boost/python/scope.hpp>
#include <boost/python/enum.hpp>

#include <osgSim/LightPoint>

using namespace boost::python;

namespace PyOSGSim {
void init_LightPoint()
{
    class_<osgSim::LightPoint> lightpoint("LightPoint");

    scope lightpoint_scope(lightpoint);

    lightpoint.def(init<const osgSim::LightPoint&>());
    lightpoint.def(init<const osg::Vec3&, const osg::Vec4& >());

    lightpoint.def_readwrite("_on", &osgSim::LightPoint::_on);
    lightpoint.def_readwrite("_position", &osgSim::LightPoint::_position);
    lightpoint.def_readwrite("_color", &osgSim::LightPoint::_color);
    lightpoint.def_readwrite("_intensity", &osgSim::LightPoint::_intensity);
    lightpoint.def_readwrite("_radius", &osgSim::LightPoint::_radius);
    lightpoint.def_readwrite("_blendingMode", &osgSim::LightPoint::_blendingMode);

#   define OSG_ENUM_MODE(VALUE) \
    (mode.value(#VALUE, osgSim::LightPoint::VALUE), \
    lightpoint.def(#VALUE, object(osgSim::LightPoint::VALUE)))

    enum_<osgSim::LightPoint::BlendingMode> mode("BlendingMode");
    OSG_ENUM_MODE(ADDITIVE);
    OSG_ENUM_MODE(BLENDED);
}
}
