// Copyright (C) 2004 Brett Hartshorn (bhartsho@yahoo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/class.hpp>
#include <boost/python/return_internal_reference.hpp>

#include <osgTerrain/HeightFieldNode>
#include "held_ptr.hpp"

using namespace boost::python;

namespace PyOSG {
void init_HeightFieldNode() {


    class_<osgTerrain::HeightFieldNode, osg::ref_ptr<osgTerrain::HeightFieldNode>, bases<osg::Group>, boost::noncopyable >
    heightFieldNode("HeightFieldNode", no_init);
    
    heightFieldNode
        .def(init<>())
        .def("traverse", &osgTerrain::HeightFieldNode::traverse)
        
        .def("setHeightField", &osgTerrain::HeightFieldNode::setHeightField)
        .def("getHeightField", (osg::HeightField*(osgTerrain::HeightFieldNode::*)())
        &osgTerrain::HeightFieldNode::getHeightField, return_internal_reference<>())
        
        .def("heightFieldHasBeenModified", &osgTerrain::HeightFieldNode::heightFieldHasBeenModified)
        
        .def("setRenderer", &osgTerrain::HeightFieldNode::setRenderer)
        .def("getRenderer", (osgTerrain::HeightFieldRenderer*(osgTerrain::HeightFieldNode::*)())
        &osgTerrain::HeightFieldNode::getRenderer, return_internal_reference<>())
        
        .def("setBaseTextureImage", &osgTerrain::HeightFieldNode::setBaseTextureImage)
        .def("getBaseTextureImage", (osg::Image* (osgTerrain::HeightFieldNode::*)())
        &osgTerrain::HeightFieldNode::getBaseTextureImage, return_internal_reference<>())
        
        .def("setDetailTextureImage", &osgTerrain::HeightFieldNode::setDetailTextureImage)
        .def("getDetailTextureImage", (osg::Image* (osgTerrain::HeightFieldNode::*)())
        &osgTerrain::HeightFieldNode::getDetailTextureImage, return_internal_reference<>())
        
        .def("setCloudShadowTextureImage", &osgTerrain::HeightFieldNode::setCloudShadowTextureImage)
        .def("getCloudShadowTextureImage", (osg::Image* (osgTerrain::HeightFieldNode::*)())
        &osgTerrain::HeightFieldNode::getCloudShadowTextureImage, return_internal_reference<>())
        
        .def("setNormalMapImage", &osgTerrain::HeightFieldNode::setNormalMapImage)
        .def("getNormalMapImage", (osg::Image* (osgTerrain::HeightFieldNode::*)())
        &osgTerrain::HeightFieldNode::getNormalMapImage, return_internal_reference<>())
        ;
}
} //namespace
