// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/def.hpp>
#include <boost/python/class.hpp>
#include <boost/python/return_internal_reference.hpp>

#include <osgUtil/DisplayRequirementsVisitor>

using namespace boost::python;

namespace {
osgUtil::DisplayRequirementsVisitor * asDisplayRequirementsVisitor(osg::NodeVisitor * nv)
{
    return dynamic_cast<osgUtil::DisplayRequirementsVisitor*>(nv);
}
} // namespace

namespace PyOSGUtil {
void init_DisplayRequirementsVisitor()
{
    def("asDisplayRequirementsVisitor", &asDisplayRequirementsVisitor, return_internal_reference<>());

    class_<osgUtil::DisplayRequirementsVisitor, bases<osg::NodeVisitor>, boost::noncopyable >("DisplayRequirementsVisitor", 
            "A visitor for traversing a scene graph establishing the OpenGL visuals are required\n"
            "to support rendering of that scene graph.  The results can then be used by\n"
            "applications to set up there windows with the correct visuals.  Have a look at\n"
            "src/osgGLUT/Viewer.cpp's Viewer::open() method for an example how to use it.\n",
            no_init)
        .def(init<>())
        .def("setDisplaySettings", &osgUtil::DisplayRequirementsVisitor::setDisplaySettings)

        .def("getDisplaySettings", &osgUtil::DisplayRequirementsVisitor::getDisplaySettings, return_internal_reference<>())
        ;
}
}
