// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
//

#include <boost/python/module.hpp>
#include <boost/python/class.hpp>
#include <boost/python/list.hpp>
#include <boost/python/extract.hpp>
#include <boost/python/iterator.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/python/return_internal_reference.hpp>
#include <boost/python/manage_new_object.hpp>
#include <boost/python/detail/api_placeholder.hpp>
#include <boost/ref.hpp>
#include <boost/lexical_cast.hpp>

#include <osgUtil/Optimizer>

#include <string>
#include <iostream>

using namespace boost::python;

namespace {
#if 0
void optimize_1(osgUtil::Optimizer * self, osg::Node* node)
{
    self->optimize(node);
}
void optimize_2(osgUtil::Optimizer * self, osg::Node* node, unsigned int options)
{
    self->optimize(node, options);
}
#endif
} // namespace

namespace PyOSGUtil {
void init_Optimizer()
{
    class_<osgUtil::Optimizer>("Optimizer")
        .def("reset", &osgUtil::Optimizer::reset)
#if 0
        .def("optimize", &optimize_1)
        .def("optimize", &optimize_2)
#endif
        // .def("optimize", &osgUtil::Optimizer::optimize)
        .def("optimize",
            (void (osgUtil::Optimizer::*)(osg::Node*))
            &osgUtil::Optimizer::optimize)
        .def("optimize",
            (void (osgUtil::Optimizer::*)(osg::Node*, unsigned int))
            &osgUtil::Optimizer::optimize)

        .def("setPermissibleOptimizationsForObject",
            &osgUtil::Optimizer::setPermissibleOptimizationsForObject)
        .def("getPermissableOptimizationsForObject",
            &osgUtil::Optimizer::getPermissibleOptimizationsForObject)

        .def("isOperationPermissibleForObject",
            (bool (osgUtil::Optimizer::*)(const osg::StateSet*, unsigned int ) const)
            &osgUtil::Optimizer::isOperationPermissibleForObject)

        .def("isOperationPermissibleForObject",
            (bool (osgUtil::Optimizer::*)(const osg::StateAttribute*, unsigned int ) const)
            &osgUtil::Optimizer::isOperationPermissibleForObject)
        ;
}
}

