// Copyright (C) 2002-2003 Gideon May (gideon@computer.org)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.

#include <boost/python/module.hpp>
#include <boost/python/class.hpp>
#include <boost/python/list.hpp>
#include <boost/python/extract.hpp>
#include <boost/python/iterator.hpp>
#include <boost/python/scope.hpp>
#include <boost/python/enum.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/python/return_internal_reference.hpp>
#include <boost/python/manage_new_object.hpp>
#include <boost/python/detail/api_placeholder.hpp>
#include <boost/ref.hpp>
#include <boost/lexical_cast.hpp>

#include <osgUtil/SceneView>
#include <osgUtil/CullVisitor>

#include <string>
#include <iostream>

#include "held_ptr.hpp"

using namespace boost::python;

namespace PyOSGUtil {
void init_SceneView3(class_<osgUtil::SceneView, osg::ref_ptr<osgUtil::SceneView>, bases<osg::Referenced>, boost::noncopyable > & sceneview)
{

    sceneview
        .def("setInitVisitor", &osgUtil::SceneView::setInitVisitor)
        .def("getInitVisitor", (osg::NodeVisitor *(osgUtil::SceneView::*)()) &osgUtil::SceneView::getInitVisitor,
                return_internal_reference<>())

        .def("setUpdateVisitor", &osgUtil::SceneView::setUpdateVisitor)
        .def("getUpdateVisitor", (osg::NodeVisitor *(osgUtil::SceneView::*)()) &osgUtil::SceneView::getUpdateVisitor,
                return_internal_reference<>())

        .def("setCullVisitor", &osgUtil::SceneView::setCullVisitor)
        .def("getCullVisitor", (osgUtil::CullVisitor *(osgUtil::SceneView::*)()) &osgUtil::SceneView::getCullVisitor,
                return_internal_reference<>())

        .def("setStateGraph", &osgUtil::SceneView::setStateGraph)
        .def("getStateGraph", (osgUtil::StateGraph *(osgUtil::SceneView::*)()) &osgUtil::SceneView::getStateGraph,
                return_internal_reference<>())

        .def("setRenderStage", &osgUtil::SceneView::setRenderStage)
        .def("getRenderStage", (osgUtil::RenderStage *(osgUtil::SceneView::*)())&osgUtil::SceneView::getRenderStage,
                return_internal_reference<>())


        .def("setCullMask", &osgUtil::SceneView::setCullMask)
        .def("getCullMask", &osgUtil::SceneView::getCullMask)

        .def("setCullMaskLeft", &osgUtil::SceneView::setCullMaskLeft)
        .def("getCullMaskLeft", &osgUtil::SceneView::getCullMaskLeft)

        .def("setCullMaskRight", &osgUtil::SceneView::setCullMaskRight)
        .def("getCullMaskRight", &osgUtil::SceneView::getCullMaskRight)

        .def("setLODScale", &osgUtil::SceneView::setLODScale,
            "Set the LOD bias for the CullVisitor to use.\n")
        
        .def("getLODScale", &osgUtil::SceneView::getLODScale,
            "Get the LOD bias.\n")

        .def("setSmallFeatureCullingPixelSize", &osgUtil::SceneView::setSmallFeatureCullingPixelSize,
            "Set the Small Feature Culling Pixel Size.\n")

        .def("getSmallFeatureCullingPixelSize", &osgUtil::SceneView::getSmallFeatureCullingPixelSize,
            "Get the Small Feature Culling Pixel Size.\n")

        ;
}
}
