// Copyright (C) 2002-2003 Brett Hartshorn (bhartsho@yahoo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
//

#include <boost/python/def.hpp>
#include <boost/python/class.hpp>
#include <boost/python/return_internal_reference.hpp>
#include <boost/ref.hpp>

#include <osgUtil/StateGraph>

using namespace boost::python;

namespace PyOSGUtil {
void init_StateGraph()
{
    class_<osgUtil::StateGraph, bases<osg::Referenced>, boost::noncopyable>
    ("StateGraph", no_init)
    .def(init<>())
    .def("setUserData", &osgUtil::StateGraph::setUserData)
    .def("getUserData", 
        (osg::Referenced*(osgUtil::StateGraph::*)())
        &osgUtil::StateGraph::getUserData, return_internal_reference<>())

    .def("empty", &osgUtil::StateGraph::empty)
    .def("leaves_empty", &osgUtil::StateGraph::leaves_empty)
    .def("getAverageDistance", &osgUtil::StateGraph::getAverageDistance)
    .def("getMinimumDistance", &osgUtil::StateGraph::getMinimumDistance)
    .def("sortFrontToBack", &osgUtil::StateGraph::sortFrontToBack)
    .def("reset", &osgUtil::StateGraph::reset)
    .def("clean", &osgUtil::StateGraph::clean)
    .def("prune", &osgUtil::StateGraph::prune)
    .def("find_or_insert", &osgUtil::StateGraph::find_or_insert,
        return_internal_reference<>())
        
    .def("addLeaf", &osgUtil::StateGraph::addLeaf)
    .def("moveStateGraph", &osgUtil::StateGraph::moveStateGraph)
    .def("moveToRootStateGraph", &osgUtil::StateGraph::moveToRootStateGraph)
    ;
}
}


