#!/usr/bin/env python

"""
A Python binding for OpenSceneGraph
"""

"""
PyOSG distutils setup script.

On Apple Mac OS X run with:
    python setup.py bdist_mpkg
On Windows run with
    python setup.py bdist_wininst
to generate an installer package.
"""

import os, shutil, sys
from distutils.core import setup, Extension
from distutils.command.build import build
from distutils.command.clean import clean
from distutils.command.install import install
from distutils.file_util import copy_file
from versions_default import *
from glob import glob
from os.path import isfile

try:
  import py2app
  import bdist_mpkg
except:
  pass

import modules

try:
    buildno = int(open('buildno.h').read().split()[2]) + 1
except:
    buildno = 1
# XXX remove me when checking in
# open('buildno.h', 'wa').write("#define BUILDNO %d\n" % buildno)

class pyosg_build(build):
    def run(self):
        destination = os.path.join(self.build_platlib, 'PyOSG')

        command = sys.executable + " scons" + os.sep + "scons.py install=" + destination
        if sys.platform != "win32": command += " -j 2"

        if os.system(command):
            print "Could not build shared PyOSG library! Executed : ", command
            sys.exit(1)

class pyosg_clean(clean):
    def run(self):
        clean.run(self)
        destination = os.path.join(self.build_lib, 'PyOSG')

        if os.system(sys.executable + " scons" + os.sep + "scons.py -c install=" + destination):
            print "Could not clean PyOSG library!"
            sys.exit(1)

class pyosg_install(install):
    def run(self):
        install.run(self)

try:
    from PyOSG.__init__ import __version__
except:
    __version__ = "Unknown"

PYOSG_CLASSIFIERS = """\
Development Status :: 5 - Beta
Intended Audience :: Developers
License :: OSI Approved :: BSD License
Programming Language :: Python
Topic :: Software Development :: Libraries :: Python Modules
Topic :: Multimedia :: Graphics
Topic :: Multimedia :: Graphics :: 3D Rendering
Operating System :: Unix
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
"""

# patch distutils if it can't cope with the "classifiers" or
# "download_url" keywords
if sys.version < '2.2.3':
    from distutils.dist import DistributionMetadata
    DistributionMetadata.classifiers = None
    DistributionMetadata.download_url = None

extensions = []
for libname in modules.ext_modules:
    cppfiles = []
    for file in os.listdir(libname):
        if file.endswith('.cpp'):
            cppfile = os.path.join(libname, file)
            cppfiles.append(cppfile)
    extensions.append(Extension("PyOSG." + '_' + libname,
                                cppfiles,
                                include_dirs=['include'],
                                define_macros=[('VERSION','"'+__version__+'"')],
                                libraries=['boost_python','Producer']+modules.ext_modules))


if sys.platform == 'darwin':
    data_files = [
        ('/usr/local/lib', ['/usr/local/lib/libboost_python-' + boost_version_default + '.dylib'])
        ]
    if 'osgSound' in modules.ext_modules:
        data_files.append( ('/usr/local/lib', ['/usr/local/lib/libfmodex.dylib']) )
    package_data = {}
elif sys.platform == 'win32':
    data_files = [
        ('.' , [boost_root_default + '\\lib\\' + 'boost_python-vc71-mt-' + boost_version_default + '.dll'])
        ]
    if 'osgSound' in modules.ext_modules:
        data_files.append( ('.', ['\\gideon\\bin\\fmodex.dll']) )
        data_files.append( ('.', ['\\gideon\\bin\\osgsound.dll']) )
    package_data = {}
else:
    data_files = []
    package_data = {}

scriptfiles = filter(isfile, glob('applications/*'))

setup(name = "PyOSG",
      version = __version__ + "-%d" % buildno,
      description = "A Python binding for OpenSceneGraph",
      long_description = __doc__,
      author = "Gideon May",
      author_email = "gideon@computer.org",
      download_url = "http://pyosg.sourceforge.net",
      url = "http://pyosg.sourceforge.net",
      cmdclass = {'build'   : pyosg_build,
                  'clean'   : pyosg_clean,
                  'install' : pyosg_install},
      ext_modules = extensions,
      py_modules  = modules.py_modules,
      data_files = data_files,
      package_data = package_data,
      classifiers = filter(None, PYOSG_CLASSIFIERS.split("\n")),
      scripts = scriptfiles,
      license = "BSD License",
      packages=['PyOSG', 'PyOSG.examples', 'PyOSG.examples.Script'],
      platforms=['Win32', 'Linux', 'Mac_OSX'],
)

